﻿/*
**
**                     MAP ZONE VIEWER - Version 1.0
**
** Map Zone Viewer loads an image file from hardrive, counts and displays
** the zones of the image divided by borderlines
**
** FreeImage library was used for the program to load images with different
** formats.
**
** Entirely designed and programmed by Caner'Trooper'Kurt on March 2021
** in Ankara, Turkey
**
** https://trooperthedreamer.com (all rights are reserved [(-:])
**
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Map Zone Viewer.h"
#include "MainWindow.h"
#include <string>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMapZoneViewer *MapZoneViewer;
//---------------------------------------------------------------------------
__fastcall TMapZoneViewer::TMapZoneViewer(TComponent* Owner)
	: TForm(Owner),
	  mMapImage(nullptr)
{
}
//---------------------------------------------------------------------------
void __fastcall TMapZoneViewer::MainMenuExitProgramClick(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------
void __fastcall TMapZoneViewer::FormCloseQuery(TObject *Sender, bool &CanClose)

{
	if(MZV::ShowQuestionBox(L"Programı sonlandırmak istediğinizden emin misiniz?",L"Çıkış?") == ID_YES) {
		CanClose = true;
	} else {
		CanClose = false;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMapZoneViewer::MainMenuAboutAboutClick(TObject *Sender)
{
	ShowInfoBox(L"Tasarım ve Programlama : Caner'Trooper'Kurt", L"Hakkında", MZV::IBT_INFO);
}
//---------------------------------------------------------------------------
void __fastcall TMapZoneViewer::FormCreate(TObject *Sender)
{
	MZV::Graphics::Initialize();
}
//---------------------------------------------------------------------------
void __fastcall TMapZoneViewer::FormDestroy(TObject *Sender)
{
	MZV::Graphics::Deinitialize();

	if(mMapImage) {
		delete mMapImage;
		mMapImage = nullptr;
	}

	if(mMap) {
		delete mMap;
		mMap = nullptr;
	}

	if(mMapZoneCounter) {
		delete mMapZoneCounter;
		mMapZoneCounter = nullptr;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMapZoneViewer::MainMenuOpenFileClick(TObject *Sender)
{
	try {
		if(MainWindowOpenPictureDialog->Execute()) {
			//clear viewport
			MapViewPort->Canvas->FillRect(Rect(0, 0, MZV::Graphics::FRAME_HORIZONTAL_PIXEL_COUNT, MZV::Graphics::FRAME_VERTICAL_PIXEL_COUNT));
			if(loadMapImage(MainWindowOpenPictureDialog->FileName.c_str())) { //load image
				int zoneCount{0};
				if(processMapImage(zoneCount)) { //count zones
					displayMapImage(zoneCount); //render the map
				}
			}
		}
	} catch (MZV::Error::Exception& ex) {
		MZV::Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());
	} catch (std::exception& ex) {
		MZV::Error::ShowError(ex.what(),L"Harita Hatası");
	} catch(Exception& exception) {
		MZV::Error::ShowError(exception.Message.c_str(),L"Harita Hatası");
	}
}
//---------------------------------------------------------------------------
void __fastcall TMapZoneViewer::FormShow(TObject *Sender)
{
	MainWindowOpenPictureDialog->InitialDir = MZV::GetApplicationDirectory().c_str();
	MainWindowOpenPictureDialog->Title      = L"Lütfen bir resim dosyası seçiniz";

	MainWindowStatusBar->Panels->Items[0]->Text = L"Sistem hazır. Dosya menüsünden harita dosyası açınız.";
}
//---------------------------------------------------------------------------
bool TMapZoneViewer::loadMapImage(const std::wstring& fileName)
{
	try {
		if(mMapImage) {
			delete mMapImage;
			mMapImage = nullptr;
		}

        MainWindowStatusBar->Panels->Items[0]->Text = L"Harita resmi yükleniyor...";

		mMapImage = new MZV::Graphics::Image(fileName, L"Harita Dosyası");

		MainWindowStatusBar->Panels->Items[0]->Text = L"Harita resmi yüklendi.";

		return true;
	} catch (MZV::Error::Exception& ex) {
		MZV::Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		MZV::Error::ShowError(ex.what(),L"Harita Dosyası Açma Hatası");

		return false;
	} catch(Exception& exception) {
		MZV::Error::ShowError(exception.Message.c_str(),L"Harita Dosyası Açma Hatası");

		return false;
	}
}
//---------------------------------------------------------------------------
bool TMapZoneViewer::processMapImage(int &zoneCount)
{
	try {
		MainWindowStatusBar->Panels->Items[0]->Text = L"Harita işleniyor...";

		mMap            = new MZV::CanerKurt::Map(mMapImage, MapViewPort);
		mMapZoneCounter = new MZV::CanerKurt::MapZoneCounter(mMap);

		zoneCount = mMapZoneCounter->Solve();

		MainWindowStatusBar->Panels->Items[0]->Text = L"Harita görüntüleniyor...";

		if(zoneCount == 0) {
			MZV::ShowInfoBox(L"Haritada hiçbir bölge saptanamadı!", L"Harita İşleme", MZV::IBT_INFO);

			return true;
		} else {
			MZV::ShowInfoBox(L"Haritada "+std::to_wstring(zoneCount)+L" bölge var.", L"Harita İşleme", MZV::IBT_INFO);
		}

		return true;
	} catch (MZV::Error::Exception& ex) {
		MZV::Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		MZV::Error::ShowError(ex.what(),L"Harita İşleme Hatası");

		return false;
	} catch(Exception& exception) {
		MZV::Error::ShowError(exception.Message.c_str(),L"Harita İşleme Hatası");

		return false;
	}
}
//---------------------------------------------------------------------------
bool TMapZoneViewer::displayMapImage(const int zoneCount) const
{
	try {
		MainWindowStatusBar->Panels->Items[0]->Text = L"Harita görüntüleniyor...";

		mMap->Show();

		MainWindowStatusBar->Panels->Items[0]->Text = (L"Harita görüntülendi. Haritada "+std::to_wstring(zoneCount)+L" alan var.").c_str();

        return true;
	} catch (MZV::Error::Exception& ex) {
		MZV::Error::ShowError(ex.getErrorMessage(), ex.getErrorTitle());

		return false;
	} catch (std::exception& ex) {
		MZV::Error::ShowError(ex.what(), L"Harita Görüntüleme Hatası");

		return false;
	} catch(Exception& exception) {
		MZV::Error::ShowError(exception.Message.c_str(),L"Harita Görüntüleme Hatası");

		return false;
	}
}
