/*
**
**                      MAP ZONE VIEWER - Version 1.0
**
** Map Zone Viewer loads an image file from hardrive, counts and displays
** the zones of the image divided by borderlines
**
** FreeImage library was used for the program to load images with different
** formats.
**
** Entirely designed and programmed by Caner'Trooper'Kurt on March 2021
** in Ankara, Turkey
**
** https://trooperthedreamer.com (all rights are reserved [(-:])
**
*/

//---------------------------------------------------------------------------
#ifndef MainWindowH
#define MainWindowH
//---------------------------------------------------------------------------
#include <System.Classes.hpp>
#include <Vcl.Controls.hpp>
#include <Vcl.StdCtrls.hpp>
#include <Vcl.Forms.hpp>
#include <Vcl.ComCtrls.hpp>
#include <Vcl.ExtCtrls.hpp>
#include <Vcl.Menus.hpp>
#include <Vcl.Dialogs.hpp>
#include <Vcl.ExtDlgs.hpp>
#include <Vcl.WinXCtrls.hpp>
#include "Map Zone Viewer.h"
//---------------------------------------------------------------------------
class TMapZoneViewer : public TForm
{
	__published: // IDE-managed Components

		TStatusBar*         MainWindowStatusBar;
		TImage*             MapViewPort;
		TMainMenu*          MainWindowMenu;
		TMenuItem*          MainMenuFile;
		TMenuItem*          MainMenuAbout;
		TMenuItem*          MainMenuOpenFile;
		TMenuItem*          N1;
		TMenuItem*          MainMenuExitProgram;
		TMenuItem*          MainMenuAboutAbout;
		TOpenPictureDialog* MainWindowOpenPictureDialog;

		void __fastcall MainMenuExitProgramClick(TObject *Sender);
		void __fastcall FormCloseQuery(TObject *Sender, bool &CanClose);
		void __fastcall MainMenuAboutAboutClick(TObject *Sender);
		void __fastcall FormCreate(TObject *Sender);
		void __fastcall FormDestroy(TObject *Sender);
		void __fastcall MainMenuOpenFileClick(TObject *Sender);
		void __fastcall FormShow(TObject *Sender);

	private:

		MZV::Graphics::Image*           mMapImage;
		MZV::CanerKurt::Map*            mMap;
		MZV::CanerKurt::MapZoneCounter* mMapZoneCounter;

	public:

		__fastcall TMapZoneViewer(TComponent* Owner);

		bool loadMapImage(const std::wstring& fileName);
		bool processMapImage(int &zoneCount);
		bool displayMapImage(const int zoneCount) const;
};
//---------------------------------------------------------------------------
extern PACKAGE TMapZoneViewer *MapZoneViewer;
//---------------------------------------------------------------------------
#endif
